"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _loaderUtils = _interopRequireDefault(require("loader-utils"));

var _path = _interopRequireDefault(require("path"));

var _resolve_file_or_module = _interopRequireDefault(require("../resolve_file_or_module"));

var _constants = require("../constants");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var _default = ({
  resolveConfig,
  isClient
}, resourceQuery) => {
  if (!/^\?/.test(resourceQuery)) {
    throw new Error(`Resource $resolve.readModelProcedure should be retrieved with resource query`);
  }

  if (isClient) {
    throw new Error(`${_constants.message.serverAliasInClientCodeError}$resolve.readModelProcedure`);
  }

  const {
    readModelName
  } = _loaderUtils.default.parseQuery(resourceQuery);

  const readModel = resolveConfig.readModels.find(({
    name
  }) => name === readModelName);
  const readModelConnector = readModel != null ? resolveConfig.readModelConnectors[readModel.connectorName] : null;
  let wrapProcedureMethodPath = null;

  for (const pathPostfix of [['es', 'wrap-procedure'], ['lib', 'wrap-procedure'], ['wrap-procedure']]) {
    try {
      wrapProcedureMethodPath = (0, _resolve_file_or_module.default)(_path.default.join(readModelConnector.module, ...pathPostfix));
    } catch (e) {}

    if (wrapProcedureMethodPath != null) {
      break;
    }
  }

  if (wrapProcedureMethodPath == null) {
    throw new Error(_constants.OPTIONAL_ASSET_ERROR);
  }

  const imports = [`import { SynchronousPromise } from 'synchronous-promise'`, `import currentReadModel from '$resolve.readModel?readModelName=${readModelName}&onlyCode=true'`, `import wrapProcedure from ${JSON.stringify(wrapProcedureMethodPath)}`];
  const constants = [`SynchronousPromise.installGlobally()`, `const procedure = wrapProcedure(currentReadModel)`];
  const exports = [`export default procedure`];
  return [...imports, ...constants, ...exports].join('\r\n');
};

exports.default = _default;
//# sourceMappingURL=$resolve.readModelProcedure.js.map